@@ -300,7 +300,8 @@ class MonitoringDriver(object):
     connectionCls = ConnectionUserAndKey
     name = None
 
-    def __init__(self, key, secret=None, secure=True, host=None, port=None):
+    def __init__(self, key, secret=None, secure=True, host=None, port=None,
+                 **kwargs):
         self.key = key
         self.secret = secret
         self.secure = secure
