@@ -13,8 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import sys
-
 try:
     import simplejson as json
 except:
@@ -30,17 +28,15 @@ from rackspace_monitoring.utils import to_underscore_separated
 from rackspace_monitoring.utils import value_to_bool
 
 from rackspace_monitoring.base import (MonitoringDriver, Entity,
-                                      NotificationPlan, MonitoringZone,
-                                      Notification, CheckType, Alarm, Check,
-                                      NotificationType, AlarmChangelog,
-                                      LatestAlarmState, Agent, AgentToken,
-                                      AgentConnection, Metric, DataPoint)
+                                       NotificationPlan, MonitoringZone,
+                                       Notification, CheckType, Alarm, Check,
+                                       NotificationType, AlarmChangelog,
+                                       LatestAlarmState, Agent, AgentToken,
+                                       AgentConnection, Metric, DataPoint)
 
 from libcloud.common.rackspace import AUTH_URL_US
 from libcloud.common.openstack import OpenStackBaseConnection
-
-API_VERSION = 'v1.0'
-API_URL = 'https://monitoring.api.rackspacecloud.com/%s' % (API_VERSION)
+from libcloud.common.openstack import OpenStackDriverMixin
 
 
 class RackspaceMonitoringValidationError(LibcloudError):
@@ -103,10 +99,10 @@ class RackspaceMonitoringResponse(Response):
         body = self.parse_body()
         if self.status == httplib.BAD_REQUEST:
             error = RackspaceMonitoringValidationError(message=body['message'],
-                                               code=body['code'],
-                                               type=body['type'],
-                                               details=body['details'],
-                                               driver=self.connection.driver)
+                                                       code=body['code'],
+                                                       type=body['type'],
+                                                       details=body['details'],
+                                                   driver=self.connection.driver)
             raise error
 
         return body
@@ -120,18 +116,20 @@ class RackspaceMonitoringConnection(OpenStackBaseConnection):
     type = Provider.RACKSPACE
     responseCls = RackspaceMonitoringResponse
     auth_url = AUTH_URL_US
-    _url_key = "monitoring_url"
 
-    def __init__(self, user_id, key, secure=False, ex_force_base_url=API_URL,
-                 ex_force_auth_url=None, ex_force_auth_version='2.0'):
-        self.api_version = API_VERSION
-        self.monitoring_url = ex_force_base_url
+    service_name = 'cloudMonitoring'
+    service_type = 'rax:monitor'
+
+    def __init__(self, user_id, key, secure=False, ex_force_base_url=None,
+                 ex_force_auth_url=None, ex_force_auth_version='2.0',
+                 ex_force_auth_token=None):
         self.accept_format = 'application/json'
         super(RackspaceMonitoringConnection, self).__init__(user_id, key,
                                 secure=secure,
                                 ex_force_base_url=ex_force_base_url,
                                 ex_force_auth_url=ex_force_auth_url,
-                                ex_force_auth_version=ex_force_auth_version)
+                                ex_force_auth_version=ex_force_auth_version,
+                                ex_force_auth_token=ex_force_auth_token)
 
     def request(self, action, params=None, data='', headers=None, method='GET',
                 raw=False):
@@ -154,7 +152,7 @@ class RackspaceMonitoringConnection(OpenStackBaseConnection):
         )
 
 
-class RackspaceMonitoringDriver(MonitoringDriver):
+class RackspaceMonitoringDriver(MonitoringDriver, OpenStackDriverMixin):
     """
     Base Rackspace Monitoring driver.
 
@@ -163,29 +161,40 @@ class RackspaceMonitoringDriver(MonitoringDriver):
     connectionCls = RackspaceMonitoringConnection
 
     def __init__(self, *args, **kwargs):
-        self._ex_force_base_url = kwargs.pop('ex_force_base_url', None)
-        self._ex_force_auth_url = kwargs.pop('ex_force_auth_url', None)
-        self._ex_force_auth_version = kwargs.pop('ex_force_auth_version', None)
+        # Remove trailing slash from the ex_force_base_url
+        if kwargs.get('ex_force_base_url', None) is not None:
+            if kwargs['ex_force_base_url'][-1] == '/':
+                kwargs['ex_force_base_url'] = kwargs['ex_force_base_url'][:-1]
+
+        OpenStackDriverMixin.__init__(self, *args, **kwargs)
         super(RackspaceMonitoringDriver, self).__init__(*args, **kwargs)
 
-        self.connection._populate_hosts_and_request_paths()
-        ep = self.connection.service_catalog.get_endpoint(name='cloudServers',
-                                               service_type='compute',
-                                               region=None)
+        self._initialize_connection_base_url(constructor_kwargs=kwargs)
 
-        tenant_id = ep['tenantId']
-        self.connection._ex_force_base_url = '%s/%s' % (
-                self.connection._ex_force_base_url, tenant_id)
+    def _initialize_connection_base_url(self, constructor_kwargs):
+        # If user specifies a custom API url (ex_force_base_url variable),
+        # don't require him to pass in tenant id and automatically retrieve it
+        # from the service catalog.
+        #
+        # Note: If user specifies both ex_force_base_url and
+        # ex_force_auth_token, ex_force_base_url must contain full URL
+        # including a tenant id.
+        ex_force_base_url = constructor_kwargs.pop('ex_force_base_url', None)
+        ex_force_auth_token = constructor_kwargs.pop('ex_force_auth_token', None)
+
+        if ex_force_base_url and not ex_force_auth_token:
+            self.connection._populate_hosts_and_request_paths()
+            ep = self.connection.service_catalog \
+                     .get_endpoint(name=self.connection.service_name,
+                                   service_type=self.connection.service_type,
+                                   region=None)
+
+            tenant_id = ep['tenantId']
+            self.connection._ex_force_base_url = '%s/%s' % (
+            self.connection._ex_force_base_url, tenant_id)
 
     def _ex_connection_class_kwargs(self):
-        rv = {}
-        if self._ex_force_base_url:
-            rv['ex_force_base_url'] = self._ex_force_base_url
-        if self._ex_force_auth_url:
-            rv['ex_force_auth_url'] = self._ex_force_auth_url
-        if self._ex_force_auth_version:
-            rv['ex_force_auth_version'] = self._ex_force_auth_version
-        return rv
+        return self.openstack_connection_kwargs()
 
     def _get_more(self, last_key, value_dict):
         key = None
@@ -750,7 +759,6 @@ class RackspaceMonitoringDriver(MonitoringDriver):
         return AgentToken(id=agent_token['id'], label=agent_token['label'],
                           token=agent_token['token'])
 
-
     def _get_installer_headers(self, resp):
         return {'shell_url': resp.headers.get('x-shell-installer-location'),
                 'json_url': resp.headers.get('location')}
@@ -763,7 +771,6 @@ class RackspaceMonitoringDriver(MonitoringDriver):
         return self._create('/agent_installers', data=data,
                           coerce=None, handler=self._get_installer_headers)
 
-
     # Agent
 
     def _to_agents(self, agent, value_dict):
